module SkirentalFrontendApi
    class Booking
        include NoBrainer::Document
        include NoBrainer::Document::Timestamps

        before_save :sync_with_backend

        has_many :payments, class_name:SkirentalPayment::Payment, dependent: :destroy

        field  :session_id,      		:type => String,                   :unique => true, :required => true,   :index => true
        field  :shop_id,         		:type => String,                                    :required => true,   :index => true
        field  :context,                :type => Hash,     :default => {}
        field  :booking_id, 			:type => String

        def voucher
            # TODO: Rethink whether this is good for the future. Future: If we want to update bookings or need
            # receipts, we need to store the booking transaction into the payment and have to handle multiple (old) payments
            confirmed_payment = self.payments.order_by(:created_at => :desc).select { |payment|  payment.voucher_code != nil }.last
            if confirmed_payment.present?
                confirmed_payment.voucher_code
            else
                nil
            end
        end

        def last_payment
            self.payments.order_by(:created_at => :desc).last
        end

        def transaction(show_tax = true)
            shop_settings = SkirentalApi::Client.settings(shop_id, "de")
            fail if context.nil?

            items = []
            start_date = Date.parse(context[:startDate]).strftime("%Y-%m-%d")
            duration = (Date.parse(context[:endDate]) - Date.parse(context[:startDate])).to_i
            currency = shop_settings[:money][:payment][:cur]
            price_total = 0
            tax_total = 0

            context[:customers].each do |customer|
                age = customer[:customerGroup][:ageFrom]
                use_insurance = customer[:use_insurance]

                customer[:packages].each do |package|
                    # First calculate the main product
                    product_name = package[:name] + (age > 0 ? " [#{customer[:customerGroup][:name]}]" : "")
                    description  = package[:description]
                    item = fetch_price_item(customer, product_name, description, package[:mainProduct][:id], start_date, duration, age, shop_id, show_tax, package[:id], currency, use_insurance)
                    next if item.nil?
                    items.push item

                    # Add Accessories
                    package[:accessories].each do |accessory|
                        product_name = accessory[:name] + (age > 0 ? " [#{customer[:customerGroup][:name]}]" : "")
                        description  = accessory[:description]
                        accessory_item = fetch_price_item(customer, product_name, description, accessory[:id], start_date, duration, age, shop_id, show_tax, package[:id], currency, use_insurance)
                        next if accessory_item.nil?
                        items.push accessory_item
                    end
                end

                customer[:products].each do |product|
                    product_name = product[:name] + (age > 0 ? " [#{customer[:customerGroup][:name]}]" : "")
                    description  = product[:description]
                    product_item = fetch_price_item(customer, product_name, description, product[:id], start_date, duration, age, shop_id, show_tax, nil, currency, use_insurance)
                    next if product_item.nil?
                    items.push product_item
                end

                self.save
            end

            items.each do |item|
                price_total += item[:price].to_d
                if show_tax
                    item[:tax] = item[:vatValue]
                    tax_total += item[:vatValue].to_d
                end
            end


            transaction = {
                :item_list => {
                    :items => items
                },
                :amount =>  {
                    :total =>  (price_total + tax_total).to_f,
                    :currency =>  currency,
                    :details => {
                        :subtotal => price_total.to_f,
                        :tax => tax_total.to_f
                    }
                },
                :description =>  "Wir bedanken uns fuer Ihre Buchung!" # TODO: do we need a description???
            }

            transaction

        end

        def sync_with_backend
            api_booking_id = SkirentalApi::Client.sync_booking_state(self)
            self.booking_id = api_booking_id if api_booking_id.present?
        end

        def fetch_price_item(customer, name, description, product_id, start_date, duration, age, shop_id, show_tax, package_id, currency, use_insurance)
            # 1. We expect that prices are independent to the lang.
            # 2. We store product_id's therefore we do not access prices of packages.
            price = SkirentalApi::Client.get_price_of_item(product_id, start_date, duration, age, shop_id, "de", package_id)
            fail if price.nil?
            return nil if price[:grossPrice].blank? || price[:grossPrice] <= 0 || price[:netPrice].blank? || price[:netPrice] <= 0

            {
                :name => name,
                :description => description,
                :sku => product_id,
                :currency => currency,
                :price => (show_tax ? (use_insurance ? price[:netPriceWithInsurance] : price[:netPrice]) : (use_insurance ? price[:grossPriceWithInsurance] : price[:grossPrice])),
                :quantity => 1,
                :age => age,
                :priceHash => price,
                :vatValue => (show_tax ? (use_insurance ? price[:vatValueWithInsurance] : price[:vatValue]) : 0)
            }
        end
    end
end
